/*-
 *  Copyright 2020 Beuth Hochschule für Technik Berlin, Hochschule für Technik Stuttgart
 * 
 *  This file is part of CityDoctor2.
 *
 *  CityDoctor2 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CityDoctor2 is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with CityDoctor2.  If not, see <https://www.gnu.org/licenses/>.
 */
package de.hft.stuttgart.citydoctor2.check.error;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.junit.Test;

import de.hft.stuttgart.citydoctor2.datastructure.ConcretePolygon;
import de.hft.stuttgart.citydoctor2.datastructure.Geometry;
import de.hft.stuttgart.citydoctor2.datastructure.GeometryType;
import de.hft.stuttgart.citydoctor2.datastructure.GmlId;
import de.hft.stuttgart.citydoctor2.datastructure.Lod;
import de.hft.stuttgart.citydoctor2.datastructure.Polygon;
import de.hft.stuttgart.quality.model.MultipleConnectedComponents;
import de.hft.stuttgart.quality.model.ValidationError;
import de.hft.stuttgart.quality.model.jaxb.Component;

public class MultipleConnectedComponentsErrorTest {

	@Test
	public void testConvertToQualityAdeDatastructure() {
		Geometry geom = new Geometry(GeometryType.SOLID, Lod.LOD2);
		GmlId geomId = new GmlId("geomId");
		geom.setGmlId(geomId);
		Polygon p1 = new ConcretePolygon();
		GmlId p1Id = new GmlId("p1Id");
		p1.setGmlId(p1Id);
		Polygon p2 = new ConcretePolygon();
		GmlId p2Id = new GmlId("p2Id");
		p2.setGmlId(p2Id);
		Polygon p3 = new ConcretePolygon();
		GmlId p3Id = new GmlId("p3Id");
		p3.setGmlId(p3Id);
		
		List<List<Polygon>> components = new ArrayList<>();
		List<Polygon> component1 = new ArrayList<>();
		component1.add(p1);
		component1.add(p2);
		components.add(component1);
		List<Polygon> component2 = new ArrayList<>();
		component2.add(p3);
		components.add(component2);
		MultipleConnectedComponentsError err = new MultipleConnectedComponentsError(geom, components);
		
		Optional<ValidationError> optional = err.convertToQualityAdeDatastructure();
		assertTrue(optional.isPresent());
		MultipleConnectedComponents validationError = (MultipleConnectedComponents) optional.get();
		assertEquals(geomId.getGmlString(), validationError.getGeometryId());
		List<Component> adeComponents = validationError.getComponents();
		assertEquals(2, adeComponents.size());
		Component adeComponent1 = adeComponents.get(0);
		assertEquals(2, adeComponent1.getPolygonIds().size());
		assertEquals(p1Id.getGmlString(), adeComponent1.getPolygonIds().get(0));
		assertEquals(p2Id.getGmlString(), adeComponent1.getPolygonIds().get(1));
		Component adeComponent2 = adeComponents.get(1);
		assertEquals(p3Id.getGmlString(), adeComponent2.getPolygonIds().get(0));
	}

}
